/*
 * SerialPort.cpp
 *
 *  Created on: 9. mars 2011
 *      Author: John Sverre Solem
 *      Based on the Serial Programming Guide for POSIX Operating Systems
 */

#include <iostream>
#include <stdio.h>
#include <string>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include "SerialPort.h"

using namespace std;

SerialPort::SerialPort(string devicename){
	this->filedescriptor = -1; // mark as not open
	this->devicename = devicename;
	open_port();
	if(isOpen()){
		configure_port();
	}
}

SerialPort::~SerialPort()
{
	if(isOpen()) close_port();
}

int SerialPort::open_port(){

	if(isOpen()) return this->filedescriptor;

	const char* dev = this->devicename.c_str();
	cout << "Attempting to open port " << dev << endl;
	int fd = open(dev, O_RDWR | O_NOCTTY | O_NDELAY);
	if(fd == -1){
		/*Unable to open serial port*/
		string error = "SerialPort::open_port: Unable to open ";
		error.append(this->devicename);
		perror(error.c_str());
		return fd;
	}else{
		this->filedescriptor = fd;
		fcntl(this->filedescriptor, F_SETFL, 0);
		return this->filedescriptor;
	}
}

void SerialPort::configure_port(){
	struct termios options;

	tcgetattr(this->filedescriptor, &options);

	// set baudrate 9600
	cfsetispeed(&options, B9600);
	cfsetospeed(&options, B9600);

	options.c_cflag |= (CLOCAL|CREAD); /* RECOMMENDED SETTINGS*/

	// no parity, 8 databits, 1 stopbit
	options.c_cflag &= ~PARENB; /* no parity */
	options.c_cflag &= ~CSTOPB; /* 1 stopbit (not 2) */
	options.c_cflag &= ~CSIZE; /* mask the character size bits */
	options.c_cflag |= CS8; /* 8 bit data*/

	tcsetattr(this->filedescriptor, TCSANOW, &options);
}

int SerialPort::port_write(const char *character){
	if(!isOpen()) return -1;
	int n = write(this->filedescriptor, character, 1);
	if(n<0){
		fputs("write() of 1 byte failed!/n", stderr);
	}
	return n;
}

int SerialPort::port_read(char* buffer, bool nodelay=true){
	if(!isOpen()) return -1;

	if(nodelay){
		fcntl(this->filedescriptor, F_SETFL, FNDELAY);
	}

	int res = read(this->filedescriptor, buffer, 1);

	if(nodelay){
		fcntl(this->filedescriptor, F_SETFL, 0);
	}

	return res;
}

void SerialPort::close_port(){
	close(this->filedescriptor);
	this->filedescriptor = -1;
}

bool SerialPort::isOpen() const{
	if(this->filedescriptor >= 0) return true;
	return false;
}

string SerialPort::toString(){
	string s = "[SerialPort,\n\t open: ";
	bool open = isOpen();
	s += (open)? "true": "false";
	s += "\n\tdevicename: ";
	s += devicename;
	s += "\n]";
	return s;
}
